<?php

declare(strict_types=1);

namespace CreoCode\ContaoTourismBundle\Picker;

use Contao\CoreBundle\Framework\FrameworkAwareInterface;
use Contao\CoreBundle\Framework\FrameworkAwareTrait;
use Contao\CoreBundle\Picker\AbstractPickerProvider;
use Contao\CoreBundle\Picker\DcaPickerProviderInterface;
use Contao\CoreBundle\Picker\PickerConfig;

use CreoCode\ContaoTourismBundle\OffersSetModel;
use CreoCode\ContaoTourismBundle\OffersModel;

class OffersPickerProvider extends AbstractPickerProvider implements DcaPickerProviderInterface, FrameworkAwareInterface
{
    use FrameworkAwareTrait;

    /**
     * {@inheritdoc}
     */
    public function getName(): string
    {
        return 'offersPicker';
    }

    /**
     * {@inheritdoc}
     */
    public function supportsContext($context): bool
    {
        return 'link' === $context && $this->getUser()->hasAccess('offers', 'modules');
    }

    /**
     * {@inheritdoc}
     */
    public function supportsValue(PickerConfig $config): bool
    {
        return false !== strpos($config->getValue(), '{{offers_url::');
    }

    /**
     * {@inheritdoc}
     */
    public function getDcaTable(): string
    {
        return 'tl_offers';
    }

    /**
     * {@inheritdoc}
     */
    public function getDcaAttributes(PickerConfig $config): array
    {
        $attributes = ['fieldType' => 'radio'];

        if ($source = $config->getExtra('source')) {
            $attributes['preserveRecord'] = $source;
        }

        if ($this->supportsValue($config)) {
            $attributes['value'] = str_replace(['{{offers_url::', '}}'], '', $config->getValue());
        }

        return $attributes;
    }

    /**
     * {@inheritdoc}
     */
    public function convertDcaValue(PickerConfig $config, $value): string
    {
        return '{{offers_url::'.$value.'}}';
    }

    /**
     * {@inheritdoc}
     */
    protected function getRouteParameters(PickerConfig $config = null): array
    {
        $params = ['do' => 'offers'];

        if (null === $config || !$config->getValue() || false === strpos($config->getValue(), '{{offers_url::')) {
            return $params;
        }

        $value = str_replace(['{{offers_url::', '}}'], '', $config->getValue());

        if (null !== ($offersSetId = $this->getOffersSetId($value))) {
            $params['table'] = 'tl_offers';
            $params['id'] = $offersSetId;
        }

        return $params;
    }

    /**
     * @param int|string $id
     */
    private function getOffersSetId($id): ?int
    {
        /** @var OffersModel $offersAdapter */
        $offersAdapter = $this->framework->getAdapter(OffersModel::class);
        $offersSetAdapter = $this->framework->getAdapter(OffersSetModel::class);

        if (!($offersModel = $offersAdapter->findById($id)) instanceof OffersModel) {
            return null;
        }

        //if (!($offersSet = $offersModel->getRelated('pid')) instanceof OffersSetModel) {
        if (!($offersSet = $offersSetAdapter->findById($offersModel->pid)) instanceof OffersSetModel) {
            return null;
        }

        return (int) $offersSet->id;
    }
}
